/* --------------------------- k_cext  ----------------------------------- */
/*                                                                              */
/* Program c directly within a pd object. */
/*                                                                              */
/* This program is free software; you can redistribute it and/or                */
/* modify it under the terms of the GNU General Public License                  */
/* as published by the Free Software Foundation; either version 2               */
/* of the License, or (at your option) any later version.                       */
/*                                                                              */
/* This program is distributed in the hope that it will be useful,              */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of               */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                */
/* GNU General Public License for more details.                                 */
/*                                                                              */
/* You should have received a copy of the GNU General Public License            */
/* along with this program; if not, write to the Free Software                  */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.  */
/*                                                                              */
/* ---------------------------------------------------------------------------- */


#include "m_pd.h"
#include "k_cext.h"

static char *version = 
"k_cext v0.2.3, written by Kjetil S. Matheussen, k.s.matheussen@notam02.no\n"
"Windows-port by Olaf Matthes.";

int instancenumber=0; // Can not be static because of the win-port.


static void k_cext_float(t_k_cext *x, t_floatarg f)
{
  /*
  int i;
  for(i=0;i<5;i++){
    post("%d: %f\n",i,x->values[i]);
  }
  */
  x->values[0]=f;

  if(x->k_cext_process!=NULL)
    (*x->k_cext_process)(x);
}

static void k_cext_bang(t_k_cext *x)
{
  k_cext_float(x,x->values[0]);
}

static t_class *k_cext_class;

static void k_cext_free(t_k_cext *x)
{
  char temp[500];

  if(x->handle!=NULL){
    k_sys_freehandle(x);
  }

  sprintf(temp,"%s.c",x->filename);
  k_sys_deletefile(temp);

  free(x->inlets);
  free(x->outlets);
  free(x->values);
}

static void *k_cext_new(t_symbol *s, t_int argc, t_atom* argv)
{
  int i;
  int lokke;
  FILE *file;
  char temp[500];
  char name[500];
  int prevwasnewline=1;
  char funcname[500];

  t_k_cext *x = (t_k_cext *)pd_new(k_cext_class);

  x->num_ins=atom_getfloatarg(0,argc,argv);
  x->num_outs=atom_getfloatarg(1,argc,argv);

  x->inlets=calloc(sizeof(t_inlet*),x->num_ins);
  x->outlets=calloc(sizeof(t_outlet*),x->num_outs);
  x->values=calloc(sizeof(t_float),x->num_ins);

  for(i=1;i<x->num_ins;i++){
    x->inlets[i-1] = floatinlet_new(&x->x_ob, &x->values[i]);
  }

  for(i=0;i<x->num_outs;i++){
    x->outlets[i] = outlet_new(&x->x_ob, gensym("float"));
  }

  for(i=2;i<argc;i++){
    char string[500];
    switch(argv[i].a_type){
    case 1:
      x->values[i-2]=atom_getfloatarg(i,argc,argv);
      break;
    case 2:
      sprintf(string,"%s",atom_getsymbolarg(i,argc,argv)->s_name);
      if(!strcmp(string,";")) goto endfor;
      break;
    }
  }
 endfor:


  k_sys_mktempfilename(x->filename);
  sprintf(name,"%s",x->filename);

  name[strlen(name)+2]=0;
  name[strlen(name)+1]='c';
  name[strlen(name)]='.';


  file=fopen(name,"w");
  //  post("name: %s\n",name);

  k_sys_writeincludes(file);

  sprintf(funcname,"k_cext_process%d",instancenumber++);

  fprintf(file,"void %s(t_k_cext *x){\n",funcname);

  for(;i<argc;i++){
    char string[500];
    switch(argv[i].a_type){
    case 1:
      fprintf(file,"%f\n",atom_getfloatarg(i,argc,argv));
      //      post("-%f-",atom_getfloatarg(i,argc,argv));
      prevwasnewline=0;
      break;
    case 2:
      sprintf(string,"%s",atom_getsymbolarg(i,argc,argv)->s_name);
      if(!strcmp(".",string)){
	sprintf(string," ");
      }
      if(!strcmp(";",string)){
	if(prevwasnewline==1) break;
	sprintf(string,";\n");
	prevwasnewline=1;
      }else{
	prevwasnewline=0;
      }
      if(!strcmp("\n",string)){
	sprintf(string," ");
      }
      fprintf(file,"%s ",string);
      //      post("-%s -",string);
      break;
    default:
      break;
    }
  }
  fprintf(file,"}\n");

  fclose(file);

  k_sys_makecompilestring(temp,name);
  post("Compiling %s",name);
  system(temp);

  sprintf(name,"%s.o",name);

  if(!k_sys_getprocessfunction(x,funcname,name)){
    post("Error in loader!");
    x->k_cext_process=NULL;
    return(NULL);
  }

  return (void *)x;
}

void k_cext_setup(void)
{

  k_sys_init();

  k_cext_class = class_new(gensym("k_cext"), (t_newmethod)k_cext_new,
			   (t_method)k_cext_free, sizeof(t_k_cext), 0, A_GIMME, 0);
  
  class_addfloat(k_cext_class, k_cext_float);
  
  class_addbang(k_cext_class, (t_method)k_cext_bang);
  
  //    class_addmethod(k_cext_class, (t_method)k_cext_ft1, gensym("ft1"), A_FLOAT, 0);
  
  class_sethelpsymbol(k_cext_class, gensym("help-k_cext.pd"));
  
  post(version);
}

